# taskman

[![standard-readme compliant](https://img.shields.io/badge/standard--readme-OK-green.svg?style=flat-square)](https://github.com/RichardLitt/standard-readme)

TODO: Put more badges here.

Taskman is a task management or todo list software manageable both in command-line and browser.

It attempts to be, but not necessarily it is at the moment:

- C-portable;
- minimal;
- low footprint.

Right now taskman consists of:

- `libtm` - a C library to manipulate tasks. Adding, removing, modifying, querying tasks.
  Backed by [SQLite](https://www.sqlite.org/index.html).
- `tm` - a command-line tool to access `libtm`. Adding, removing, modifying, querying tasks.
- `tms` - a HTTP server, backed by [Mongoose](https://github.com/cesanta/mongoose) (HTTP server)
  and [jansson](https://jansson.readthedocs.io/) (JSON). Serves:

  - RPC endpoints for adding, removing, modifying, querying tasks.
  - Very basic website.

Uninteresting stuff:

- Builds with Meson, and thus Ninja.
- Builds & runs on my Debian GNU/Linux, should not be Windows-ready.
- Right now tasks are only... ID and name.
- `libtm` has a C test suite that links directly, powered by [Criterion](https://github.com/Snaipe/Criterion).
- `tm` has a C test suite that runs the tool and verifies output - Criterion too.
- About `tms`:

  - C test suite that runs the tool, verifies outputs and verifies HTTP responses. Criterion.
  - Javascript files are tested by [jest](https://github.com/facebook/jest/).
  - Javascript files are "compiled" against [Browserify](http://browserify.org/) and
    [babelify](https://github.com/babel/babelify). Because I like `require()` and I'd like
    javascripts to work on most platforms.
  - Static content for `tms` is converted to C files via internal tool `file-to-c`,
     and included in the binary. Has C test suite of course. Criterion?

- Lots of continuous integration scripts to ease work:

  - `ci/build-gcc.sh` builds and runs tests;
  - `ci/analysis.sh` runs static code analysis tools against everything;
  - `ci/coverage.sh` prints code coverage for C test suites;
  - `ci/leaks.sh` checks C tests for leaks.

## Table of Contents

- [Security](#security)
- [Install](#install)
- [Usage](#usage)
- [API](#api)
- [Maintainers](#maintainers)
- [Contribute](#contribute)
- [License](#license)

## Security

These security measures are NOT present in taskman:

- accounts, passwords, cookies, sessions;
- database encryptions;
- obfuscations of code;
- SSL/TLS/HTTPS - server should be setup behind a reverse proxy;
- cookies.

Consider this WIP and do not leave it exposed to Internet.
Also GDPR impact is not known nor analyzed at the moment, so consider taskman to be non-compliant.

## Install

Have:

* [Meson](http://mesonbuild.com/);
* [Node.js](https://nodejs.org/en/), and thus [npm](https://www.npmjs.com/).
  Required only for `tms`.

All other dependencies will be automatically downloaded.

```bash
meson build && cd build && ninja install
```

## Usage

### Command-line

Add tasks:
```
tm a "Buy milk"
tm add "Buy milk"
```

List/query tasks:

```
tm q
tm query

# By name
tm q -n milk
tm q --name milk

# By ID
tm q -i 1
tm q --id 1
```

Modify tasks. Use same filters as for `tm q`:

```
tm m -i 1 "Buy beer"
tm modify -n milk "Buy beer"
```

Remove tasks. Use same filters as for `tm q` and `tm m`:

```
tm r -i 1
rm remove -n beer
```

### Server

Start server on default IP 127.0.0.1 and port 3805:

```
tms
tms -i 127.0.0.1 -p 3805
```

## API

### Library

TODO. Consult `libtm/test/*.c` tests for usage.

### Server

Adding tasks:

```
POST /add
Content-Type: application/json

{
    "name": "milk"
}

HTTP/1.1 200 OK
Content-Length: 0

```

Listing/querying tasks:

```
POST /query
Content-Type: application/json

{
    "id-equals": 1,
    "name-contains": "milk"
}

HTTP/1.1 200 OK
Content-Type: application/json

[
    {"id": 1, "name": "Buy milk"},
    ...
]
```

Or...

```
GET /query
Content-Type: application/json


HTTP/1.1 200 OK
Content-Type: application/json

[
    ...
]
```

Modifying tasks:

```
POST /modify
Content-Type: application/json

{
    "query":
    {
        "id-equals": 1,
        "name-contains": "milk"
    },
    "args":
    {
        "name": "Buy beer"
    }
}

HTTP/1.1 200 OK
Content-Length: 0

```

Removing tasks:

```
POST /remove
Content-Type: application/json

{
    "id-equals": 1,
    "name-contains": "milk"
}

HTTP/1.1 200 OK
Content-Length: 0

```

## Tests and other continuous integration stuff

Have:

- tests:

  - [Criterion](https://github.com/Snaipe/Criterion) - "native";
  - [jest](https://github.com/facebook/jest/) - javascript.

- code analysis:

  - [Cppcheck](http://cppcheck.sourceforge.net/) - C,
  - [Vera++](https://bitbucket.org/verateam/vera/wiki/Home) - C,
  - [GNU Complexity](https://www.gnu.org/software/complexity/) - C,
  - [ESLint](https://eslint.org/) - javascript,
  - [HTMLHint](https://github.com/thedaviddias/HTMLHint) - html files,
  - [pymarkdownlint](https://github.com/jorisroovers/pymarkdownlint) - internal Python scripts,
  - [Alex](http://alexjs.com/) - this README and other markdown files,
  - [markdown-spellcheck](https://www.npmjs.com/package/markdown-spellcheck) - this README and other markdown files.

- C code coverage:

  - [Gcovr](https://gcovr.com/).

- C leak checks:

  - [Valgrind](http://valgrind.org/).

## Maintainers

[@tsieprawski](https://bitbucket.org/tsieprawski)

## Contribute

PRs accepted.

Small note: If editing the README, please conform to the [standard-readme](https://github.com/RichardLitt/standard-readme) specification.

## License

MIT © 2018 Tomasz Sieprawski
