/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <errno.h>
#include <stdio.h>
#include <string.h>

int main(int argc, char **argv)
{
    FILE *ifile = 0;
    FILE *cfile = 0;
    unsigned long size = 0;

    if (4 != argc)
    {
        fprintf(stderr, "Usage: %s [input] [variable] [cfile]\n\n",
            argv[0]);
        return 1;
    }

    ifile = fopen(argv[1], "rb");
    if (0 == ifile)
    {
        fprintf(stderr, "Cannot open file for reading: %s\n",
            strerror(errno));
        return 1;
    }

    cfile = fopen(argv[3], "wb");
    if (0 == cfile)
    {
        fclose(ifile);
        fprintf(stderr, "Cannot open file for writing: %s\n",
            strerror(errno));
        return 1;
    }
    fprintf(cfile, "char %s[] = {", argv[2]);

    while (0 == feof(ifile))
    {
        char c;
        if (0 == fread(&c, 1, 1, ifile))
        {
            break;
        }
        fprintf(cfile, "0x%.2X,", (int)c);
        size++;
    }
    fclose(ifile);
    fprintf(cfile, "};\n");
    fprintf(cfile, "unsigned long %s_size = %lu;\n", argv[2], size);
    fclose(cfile);
    return 0;
}
