/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <stdio.h>

#include <criterion/criterion.h>

#include "common.h"

static char *expected_usage = "Usage: ./file-to-c [input] [variable] [cfile]\n\n";

TestSuite(file_to_c);

Test(file_to_c, invalid_no_args)
{
    char *argv[2];
    argv[0] = "./file-to-c";
    argv[1] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(file_to_c, invalid_no_variable)
{
    char *argv[3];
    argv[0] = "./file-to-c";
    argv[1] = "input.txt";
    argv[2] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(file_to_c, invalid_no_cfile)
{
    char *argv[4];
    argv[0] = "./file-to-c";
    argv[1] = "input.txt";
    argv[2] = "content_input";
    argv[3] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(file_to_c, converts)
{
    char *argv[5];
    FILE *ifile = 0;

    ifile = fopen("input.txt", "wb");
    if (0 == ifile)
    {
        cr_expect_fail("Cannot open file for writing");
        return;
    }

    fclose(ifile);

    argv[0] = "./file-to-c";
    argv[1] = "input.txt";
    argv[2] = "content_input";
    argv[3] = "output.c";
    argv[4] = 0;

    run_with_args(argv, 0, "", 0, "", 0);
    expect_file("output.c");
}
