#!/usr/bin/python3
"""
    Runs meson tests against valgrind, with trace children on
    and exits 1 if any child leaked.
"""

from os import environ, remove, makedirs
import os.path
import re
import shutil
import subprocess

from glob import glob
from retry import retry

for old_memcheck_log in glob('{}/memcheck.*.log'.format(environ['MESON_BUILD_ROOT'])):
    remove(old_memcheck_log)

ARGS = ['meson', 'test', '--verbose',
        '--wrap=valgrind ' + \
        '--log-file=memcheck.%p.%n.log ' + \
        '--leak-check=full ' + \
        '--show-leak-kinds=all ' + \
        '--leak-resolution=high ' + \
        '--num-callers=20 ' + \
        '--show-reachable=yes ' + \
        '--trace-children=yes'
       ]
RET = subprocess.Popen(ARGS, cwd=environ['MESON_BUILD_ROOT']).wait()
RESULTS = glob('{}/memcheck.*.log'.format(environ['MESON_BUILD_ROOT']))

@retry(tries=10, delay=1)
def interpret_results():
    """
        @todo
    """
    failures = []
    pairs = [(afile, open(afile).readlines()) for afile in RESULTS]
    pattern = re.compile("ERROR SUMMARY: [1-9]")
    for afile, alog in pairs:
        found = False
        for line in alog:
            if pattern.search(line):
                found = True
                break

        if found:
            failures.append(afile)
        else:
            remove(afile)

    if failures:
        archive_path = os.path.join(environ['MESON_BUILD_ROOT'], 'leaks')
        shutil.rmtree(archive_path, ignore_errors=True)
        makedirs(archive_path, exist_ok=True)

        for afile in failures:
            print(afile)
            shutil.copy(afile, archive_path)

        exit(1)

interpret_results()
