/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"

void tm_compact_sprintf(char **pthis, unsigned int *plen,
    tm_compact_sprint_length_f length_f, tm_compact_sprint_do_f do_f,
    void *userdata)
{
    char *now = 0;
    unsigned int add_length = 0;
    unsigned int length;
    unsigned int offset = 0;
    char *othis = 0;
    unsigned int olen = 0;
    /* LCOV_EXCL_START */
    if (0 != *pthis)
    {
        othis = *pthis;
        olen = *plen;
        offset = olen;
    }
    /* LCOV_EXCL_STOP */
    (length_f)(userdata, &add_length);
    length=offset + add_length;
    now=(char *)malloc(sizeof(char) * (length + 1));
    /* LCOV_EXCL_START */
    if (0 != othis)
    {
        memcpy(now, othis, olen * sizeof(char));
        free(othis);
    }
    /* LCOV_EXCL_STOP */
    (do_f)(userdata, now + offset);
    now[length] = 0;
    *plen=length;
    *pthis=now;
}

/*
void tm_compact_sprintui_length(void *userdata, unsigned int *plength)
{
    char buf[14];
    unsigned int v = *((unsigned int *)userdata);
    unsigned int len = sprintf(buf, "%u", v);
    *plength = len;
}

void tm_compact_sprintui_do(void *userdata, char *value)
{
    unsigned int v = *((unsigned int *)userdata);
    sprintf(value, "%u", v);
}

void tm_compact_sprintui(char **pthis, unsigned int *plen,
    unsigned int value)
{
    tm_compact_sprintf(pthis, plen,
                       tm_compact_sprintui_length,
                       tm_compact_sprintui_do,
                       (void *)&value);
}

void tm_compact_sprintp_length(void *userdata, unsigned int *plength)
{
    char buf[14];
    void *v = userdata;
    unsigned int len = sprintf(buf, "%p", v);
    *plength = len;
}

void tm_compact_sprintp_do(void *userdata, char *value)
{
    void *v = userdata;
    sprintf(value, "%p", v);
}

void tm_compact_sprintp(char **pthis, unsigned int *plen,
    void *value)
{
    tm_compact_sprintf(pthis, plen,
                       tm_compact_sprintp_length,
                       tm_compact_sprintp_do,
                       value);
}
*/

typedef struct tm_compact_sprints_value
{
    unsigned int len;
    char *value;
} tm_compact_sprints_value;

void tm_compact_sprints_length(void *userdata, unsigned int *plength)
{
    tm_compact_sprints_value *v = (tm_compact_sprints_value *)userdata;
    *plength = v->len;
}

void tm_compact_sprints_do(void *userdata, char *value)
{
    tm_compact_sprints_value *v = (tm_compact_sprints_value *)userdata;
    memcpy(value, v->value, sizeof(char) * v->len);
}

void tm_compact_sprints(char **pthis, unsigned int *plen,
    char *value)
{
    tm_compact_sprints_value v = {strlen(value), value};
    tm_compact_sprintf(pthis, plen,
                       tm_compact_sprints_length,
                       tm_compact_sprints_do,
                       (void *)&v);
}
