/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <stdlib.h>
#include <string.h>

#include "tm.h"
#include "query.h"

int tm_query_name_contains(tm *this, char *name, tm_query **pthis)
{
    if (0 == this || 0 == name || 0 == pthis || 0 != *pthis)
    {
        return 0;
    }

    return tm_storage_query_name_contains(this->storage, name, pthis);
}

int tm_query_id_equals(tm *this, unsigned long id, tm_query **pthis)
{
    if (0 == this || 0 == pthis || 0 != *pthis)
    {
        return 0;
    }

    return tm_storage_query_id_equals(this->storage, id, pthis);
}

int tm_query_and(tm *this, unsigned int clen, tm_query **cs, tm_query **pthis)
{
    if (0 == this || 0 == cs || 0 == pthis || 0 != *pthis)
    {
        return 0;
    }

    return tm_storage_query_and(this->storage, clen, cs, pthis);
}

int tm_query_new(void *data, tm_query_free_f free_f, tm_query **pthis)
{
    tm_query *this = 0;
    this = (tm_query *)malloc(sizeof(tm_query));
    this->data = data;
    this->free_f = free_f;
    *pthis = this;
    return 1;
}

int tm_query_free(tm_query **pthis)
{
    tm_query *this = 0;
    if (0 == pthis || 0 == *pthis)
    {
        return 0;
    }

    this = *pthis;
    (this->free_f)(this->data);
    free(this);
    *pthis = 0;
    return 1;
}
