/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#ifndef HAVE_PRV_STORAGE
#define HAVE_PRV_STORAGE

#include <tm.h>

typedef void (*tm_storage_free_f)(tm_storage *);
typedef int (*tm_storage_add_tasks_f)(tm_storage *, tm_task **tasks, unsigned int len);
typedef int (*tm_storage_modify_tasks_f)(tm_storage *, tm_task **tasks, unsigned int len);
typedef int (*tm_storage_remove_tasks_f)(tm_storage *, tm_task **tasks, unsigned int len);
typedef int (*tm_storage_run_query_f)(tm_storage *, tm_query *, tm_task ***parr, unsigned int *plen);
typedef int (*tm_storage_query_name_contains_f)(tm_storage *, char *, tm_query **);
typedef int (*tm_storage_query_id_equals_f)(tm_storage *, unsigned long, tm_query **);
typedef int (*tm_storage_query_and_f)(tm_storage *, unsigned int, tm_query **, tm_query **);

struct tm_storage
{
    void *data;
    tm_storage_free_f free_f;
    tm_storage_run_query_f run_query_f;
    tm_storage_add_tasks_f add_tasks_f;
    tm_storage_modify_tasks_f modify_tasks_f;
    tm_storage_remove_tasks_f remove_tasks_f;
    tm_storage_query_name_contains_f query_name_contains_f;
    tm_storage_query_id_equals_f query_id_equals_f;
    tm_storage_query_and_f query_and_f;
};

int tm_storage_free(tm_storage **);
int tm_storage_add_tasks(tm_storage *this, tm_task **tasks, unsigned int len);
int tm_storage_modify_tasks(tm_storage *this, tm_task **tasks, unsigned int len);
int tm_storage_remove_tasks(tm_storage *this, tm_task **tasks, unsigned int len);
int tm_storage_run_query(tm_storage *this, tm_query *, tm_task ***parr, unsigned int *plen);
int tm_storage_query_name_contains(tm_storage *this, char *, tm_query **);
int tm_storage_query_id_equals(tm_storage *this, unsigned long, tm_query **);
int tm_storage_query_and(tm_storage *this, unsigned int, tm_query **, tm_query **);
#endif
