/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <criterion/criterion.h>

#include <tm.h>

TestSuite(tm);

Test(tm, happy_path)
{
    tm *this = 0;
    tm_storage *storage = 0;

    cr_expect_eq(1, tm_storage_sqlite("tm_happy_path.db", &storage));
    cr_assert_neq(0, storage);
    cr_expect_eq(1, tm_new(storage, &this));
    cr_assert_neq(0, this);

    cr_expect_eq(1, tm_free(&this));
    cr_assert_eq(0, this);
    remove("tm_happy_path.db");
}

Test(tm, wrong_usage)
{
    tm *this = (tm *)1;
    tm_storage *storage = (tm_storage *)1;
    tm_query *query = (tm_query *)1;
    tm_query **queries = (tm_query **)1;
    tm_task **tasks = 0;
    tm_task *task = (tm_task *)1;
    char *name = 0;

    cr_expect_eq(0, tm_new(0, 0));
    cr_expect_eq(0, tm_new(storage, &this));

    cr_expect_eq(0, tm_storage_sqlite(0, 0));
    cr_expect_eq(0, tm_storage_sqlite(0, &storage));
    cr_expect_eq(0, tm_storage_sqlite("", 0));

    cr_expect_eq(0, tm_run_query(0, 0, 0, 0));
    cr_expect_eq(0, tm_run_query(this, 0, 0, 0));
    cr_expect_eq(0, tm_run_query(0, query, 0, 0));
    cr_expect_eq(0, tm_run_query(this, query, 0, 0));
    cr_expect_eq(0, tm_run_query(0, 0, &tasks, 0));
    cr_expect_eq(0, tm_run_query(this, 0, &tasks, 0));
    cr_expect_eq(0, tm_run_query(0, query, &tasks, 0));

    cr_expect_eq(0, tm_query_free(0));
    query = 0;
    cr_expect_eq(0, tm_query_free(&query));
    query = (tm_query *)1;

    cr_expect_eq(0, tm_query_name_contains(0, 0, 0));
    cr_expect_eq(0, tm_query_name_contains(this, 0, 0));
    cr_expect_eq(0, tm_query_name_contains(0, "", 0));
    cr_expect_eq(0, tm_query_name_contains(this, "", 0));
    cr_expect_eq(0, tm_query_name_contains(0, 0, &query));
    cr_expect_eq(0, tm_query_name_contains(this, 0, &query));
    cr_expect_eq(0, tm_query_name_contains(0, "", &query));

    cr_expect_eq(0, tm_query_id_equals(0, 0, 0));
    cr_expect_eq(0, tm_query_id_equals(this, 0, 0));
    cr_expect_eq(0, tm_query_id_equals(0, 0, &query));

    cr_expect_eq(0, tm_query_and(0, 0, 0, 0));
    cr_expect_eq(0, tm_query_and(this, 0, 0, 0));
    cr_expect_eq(0, tm_query_and(0, 0, queries, 0));
    cr_expect_eq(0, tm_query_and(this, 0, queries, 0));
    cr_expect_eq(0, tm_query_and(0, 0, 0, &query));
    cr_expect_eq(0, tm_query_and(this, 0, 0, &query));
    cr_expect_eq(0, tm_query_and(0, 0, queries, &query));

    cr_expect_eq(0, tm_task_new(0));
    cr_expect_eq(0, tm_task_new(&task));

    cr_expect_eq(0, tm_task_get_id(0, 0));

    cr_expect_eq(0, tm_task_set_id(0, 0));

    cr_expect_eq(0, tm_task_get_name(0, 0));
    cr_expect_eq(0, tm_task_get_name(task, 0));
    cr_expect_eq(0, tm_task_get_name(0, &name));

    cr_expect_eq(0, tm_task_set_name(0, 0));
    cr_expect_eq(0, tm_task_set_name(task, 0));
    cr_expect_eq(0, tm_task_set_name(0, ""));

    cr_expect_eq(0, tm_task_free(0));
    task = 0;
    cr_expect_eq(0, tm_task_free(&task));

    cr_expect_eq(0, tm_task_arr_free(0, 0));

    cr_expect_eq(0, tm_task_arr_set_name(0, 0, 0));
    cr_expect_eq(0, tm_task_arr_set_name(tasks, 0, 0));
    cr_expect_eq(0, tm_task_arr_set_name(0, 0, ""));

    cr_expect_eq(0, tm_add_tasks(0, 0, 0));
    cr_expect_eq(0, tm_add_tasks(0, tasks, 0));
    cr_expect_eq(0, tm_add_tasks(this, 0, 0));

    cr_expect_eq(0, tm_modify_tasks(0, 0, 0));
    cr_expect_eq(0, tm_modify_tasks(0, tasks, 0));
    cr_expect_eq(0, tm_modify_tasks(this, 0, 0));

    cr_expect_eq(0, tm_remove_tasks(0, 0, 0));
    cr_expect_eq(0, tm_remove_tasks(0, tasks, 0));
    cr_expect_eq(0, tm_remove_tasks(this, 0, 0));

    this = 0;
    cr_expect_eq(0, tm_free(0));
    cr_expect_eq(0, tm_free(&this));
}
