/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <criterion/criterion.h>

#include "common.h"

TestSuite(add);

Test(add, one)
{
    init_folder();
    do_add(1, "Buy milk");
    do_query("ID | Name\n1  | Buy milk\n", 0);
    do_query("ID | Name\n1  | Buy milk\n", 2, "-n", "Buy milk");
    clear_folder();
}

Test(add, many_dbfiles)
{
    init_folder();
    do_add(1, "Buy milk");
    do_add_dbfile("a.db", 1, "Buy beer");

    do_query("ID | Name\n1  | Buy milk\n", 0);
    do_query_dbfile("a.db", "ID | Name\n1  | Buy beer\n", 0);

    remove("a.db");
    clear_folder();
}

Test(add, same)
{
    init_folder();
    do_add(1, "Buy milk");
    do_add(1, "Buy milk");
    do_query("ID | Name\n1  | Buy milk\n2  | Buy milk\n", 0);
    clear_folder();
}

Test(add, three)
{
    init_folder();
    do_add(1, "Buy milk");
    do_add(1, "Buy gas");
    do_add(1, "Buy beer");
    do_query("ID | Name\n1  | Buy milk\n2  | Buy gas\n3  | Buy beer\n", 0);
    do_query("ID | Name\n1  | Buy milk\n2  | Buy gas\n3  | Buy beer\n", 2, "-n", "Buy");
    do_query("ID | Name\n1  | Buy milk\n", 2, "-n", "milk");
    do_query("ID | Name\n2  | Buy gas\n", 2, "-n", "gas");
    do_query("ID | Name\n3  | Buy beer\n", 2, "-n", "beer");
    clear_folder();
}
