/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <criterion/criterion.h>

#include "common.h"

static char *expected_usage = "Usage: ./tm [-h]\n"
                              "       ./tm [opts] a{dd} [name]\n"
                              "       ./tm [opts] q{uery} [filters]\n"
                              "       ./tm [opts] r{emove} [filters]\n"
                              "       ./tm [opts] m{odify} [filters] [name]\n"
                              "\n"
                              "Options:\n"
                              "       -d/--dbfile dbfile. Default: tm.db\n"
                              "\n"
                              "Filters:\n"
                              "       -i/--id id\n"
                              "       -n/--name name\n"
                              "\n";

TestSuite(usage);

Test(usage, no_args)
{
    char *argv[2];
    argv[0] = "./tm";
    argv[1] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, unknown_command)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "kra";
    argv[2] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, help_long)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "--help";
    argv[2] = 0;

    run_with_args(argv, 0, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, help_short)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "-h";
    argv[2] = 0;

    run_with_args(argv, 0, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, dbfile_short_no_arg)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "-d";
    argv[2] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, dbfile_long_no_arg)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "--dbfile";
    argv[2] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, add_short_no_arg)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "a";
    argv[2] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, add_long_no_arg)
{
    char *argv[3];
    argv[0] = "./tm";
    argv[1] = "add";
    argv[2] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, modify_short_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "-i";
    argv[3] = "0";
    argv[4] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, modify_short_no_value2)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "--";
    argv[3] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, modify_short_no_value3)
{
    char *argv[6];
    argv[0] = "./tm";
    argv[1] = "m";
    argv[2] = "-i";
    argv[3] = "0";
    argv[4] = "--";
    argv[5] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, modify_long_no_value)
{
    char *argv[5];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "-i";
    argv[3] = "0";
    argv[4] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, modify_long_no_value2)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "--";
    argv[3] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, modify_long_no_value3)
{
    char *argv[6];
    argv[0] = "./tm";
    argv[1] = "modify";
    argv[2] = "-i";
    argv[3] = "0";
    argv[4] = "--";
    argv[5] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, query_short_unexpected_argument)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "-x";
    argv[3] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, query_short_unfinished_argument)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "q";
    argv[2] = "-n";
    argv[3] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, query_long_unexpected_argument)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "-x";
    argv[3] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}

Test(usage, query_long_unfinished_argument)
{
    char *argv[4];
    argv[0] = "./tm";
    argv[1] = "query";
    argv[2] = "-n";
    argv[3] = 0;

    run_with_args(argv, 1, "", 0, expected_usage, strlen(expected_usage));
}
