/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <criterion/criterion.h>
#include <jansson.h>

#include "common.h"

TestSuite(add);

Test(add, one)
{
    query_row milk = {1, "Buy milk"};

    expect_process_started();
    do_add("Buy milk");
    do_query(1, 0,
        &milk);
    do_query(1, 1,
        &milk,
        "id-equals", 1);
    do_query(1, 1,
        &milk,
        "name-contains", "Buy milk");
    do_query(1, 2,
        &milk,
        "id-equals", 1,
        "name-contains", "Buy milk");
    expect_process_died();
}

Test(add, same)
{
    query_row milk1 = {1, "Buy milk"};
    query_row milk2 = {2, "Buy milk"};

    expect_process_started();
    do_add("Buy milk");
    do_add("Buy milk");
    do_query(2, 0,
        &milk1, &milk2);
    expect_process_died();
}

Test(add, three)
{
    query_row milk = {1, "Buy milk"};
    query_row gas = {2, "Buy gas"};
    query_row beer = {3, "Buy beer"};

    expect_process_started();
    do_add("Buy milk");
    do_add("Buy gas");
    do_add("Buy beer");
    do_query(3, 0,
        &milk, &gas, &beer);
    do_query(3, 1,
        &milk, &gas, &beer,
        "name-contains", "Buy");
    do_query(1, 1,
        &milk,
        "name-contains", "milk");
    do_query(1, 1,
        &gas,
        "name-contains", "gas");
    do_query(1, 1,
        &beer,
        "name-contains", "beer");
    expect_process_died();
}

Test(add, invalid_name_not_string)
{
    char *request_str = 0;
    json_t *request_json = json_object();
    json_t *tmp = json_integer(0);
    json_object_set(request_json, "name", tmp);
    json_decref(tmp);
    request_str = json_dumps(request_json, JSON_COMPACT);
    json_decref(request_json);

    expect_process_started();
    expect_http_post("http://127.0.0.1:3805/add", request_str, 400, 0);
    free(request_str);
    expect_process_no_output();
    expect_process_died();
}

Test(add, invalid_empty)
{
    expect_process_started();
    expect_http_post("http://127.0.0.1:3805/add", "{}", 400, 0);
    expect_process_no_output();
    expect_process_died();
}

Test(add, invalid_unknown_args)
{
    char *request_str = 0;
    json_t *request_json = json_object();
    json_t *tmp = json_integer(0);
    json_object_set(request_json, "unknown-arg", tmp);
    json_decref(tmp);
    request_str = json_dumps(request_json, JSON_COMPACT);
    json_decref(request_json);

    expect_process_started();
    expect_http_post("http://127.0.0.1:3805/add", request_str, 400, 0);
    free(request_str);
    expect_process_no_output();
    expect_process_died();
}

Test(add, invalid_not_json)
{
    expect_process_started();
    expect_http_post("http://127.0.0.1:3805/add", "}", 400, 0);
    expect_process_no_output();
    expect_process_died();
}

Test(add, invalid_not_post)
{
    expect_process_started();
    expect_http_get("http://127.0.0.1:3805/add", 405, 0);
    expect_process_no_output();
    expect_process_died();
}
