/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <criterion/criterion.h>
#include <jansson.h>

#include "common.h"

TestSuite(query);

Test(query, empty)
{
    expect_process_started();

    do_query(0, 0);
    do_query(0, 1,
        "id-equals", 1);
    do_query(0, 1,
        "name-contains", "Buy beer");

    do_remove(0, 1,
        "id-equals", 1);
    do_remove(0, 1,
        "name-contains", "Buy beer");

    do_modify(0, 0, 1,
        "name", "Buy beer");
    do_modify(0, 1, 1,
        "id-equals", 1,
        "name", "Buy beer");
    do_modify(0, 1, 1,
        "name-contains", "milk",
        "name", "Buy beer");

    expect_process_died();
}

Test(query, invalid_id_equals_not_int)
{
    char *request_str = 0;
    json_t *request_json = json_object();
    json_t *tmp = json_string("invalid");
    json_object_set(request_json, "id-equals", tmp);
    json_decref(tmp);
    request_str = json_dumps(request_json, JSON_COMPACT);
    json_decref(request_json);

    expect_process_started();
    expect_http_post("http://127.0.0.1:3805/query", request_str, 400, 0);
    free(request_str);
    expect_process_no_output();
    expect_process_died();
}

Test(query, invalid_name_contains_not_string)
{
    char *request_str = 0;
    json_t *request_json = json_object();
    json_t *tmp = json_integer(0);
    json_object_set(request_json, "name-contains", tmp);
    json_decref(tmp);
    request_str = json_dumps(request_json, JSON_COMPACT);
    json_decref(request_json);

    expect_process_started();
    expect_http_post("http://127.0.0.1:3805/query", request_str, 400, 0);
    free(request_str);
    expect_process_no_output();
    expect_process_died();
}

Test(query, invalid_unknown_arg)
{
    char *request_str = 0;
    json_t *request_json = json_object();
    json_t *tmp = json_integer(0);
    json_object_set(request_json, "unknown-arg", tmp);
    json_decref(tmp);
    request_str = json_dumps(request_json, JSON_COMPACT);
    json_decref(request_json);

    expect_process_started();
    expect_http_post("http://127.0.0.1:3805/query", request_str, 400, 0);
    free(request_str);
    expect_process_no_output();
    expect_process_died();
}

Test(query, invalid_not_json)
{
    expect_process_started();
    expect_http_post("http://127.0.0.1:3805/query", "}", 400, 0);
    expect_process_no_output();
    expect_process_died();
}
