/*
 * Copyright (c) 2018 Tomasz Sieprawski
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <signal.h>

#include <criterion/criterion.h>

#include "common.h"

TestSuite(usage);

Test(usage, help_long)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "--help";
    argv[2] = 0;

    expect_process_usage(argv, 0);
}

Test(usage, help_short)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "-h";
    argv[2] = 0;

    expect_process_usage(argv, 0);
}

Test(usage, ip_short_unexpected_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "-i";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}
Test(usage, ip_long_unexpected_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "--ip";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, port_short_unfinished_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "-p";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, port_long_unfinished_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "--port";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, dbfile_short_unfinished_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "-d";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, dbfile_long_unfinished_argument)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "--dbfile";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, unknown_option)
{
    char *argv[3];
    argv[0] = "./tms";
    argv[1] = "unknown";
    argv[2] = 0;

    expect_process_usage(argv, 1);
}

Test(usage, starts)
{
    char *argv[2];

    argv[0] = "./tms";
    argv[1] = 0;

    expect_process_started_and_died(argv);
}

Test(usage, starts_custom_ip)
{
    char *argv[4];

    argv[0] = "./tms";
    argv[1] = "-i";
    argv[2] = "127.0.0.1";
    argv[3] = 0;

    expect_process_started_and_died(argv);
}

Test(usage, starts_custom_port)
{
    char *argv[4];

    argv[0] = "./tms";
    argv[1] = "-p";
    argv[2] = "3805";
    argv[3] = 0;

    expect_process_started_and_died(argv);
}

Test(usage, starts_custom_dbfile)
{
    char *argv[4];

    argv[0] = "./tms";
    argv[1] = "-d";
    argv[2] = "tm.db";
    argv[3] = 0;

    expect_process_started_and_died(argv);
}

Test(usage, starts_accesslog)
{
    expect_process_started_accesslog();

    expect_http_get("http://127.0.0.1:3805/ping", 200, 0);
    expect_process_output_accesslog("127.0.0.1", "GET", "/ping", 200, 0);

    expect_http_post("http://127.0.0.1:3805/query", "{}", 200, "[]");
    expect_process_output_accesslog("127.0.0.1", "POST", "/query", 200, 2);

    expect_http_get("http://127.0.0.1:3805/unknown", 404, 0);
    expect_process_output_accesslog("127.0.0.1", "GET", "/unknown", 404, 0);

    expect_process_died();
}
